package com.mortals.xhx.module.product.web;

import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.model.ProductVersionEntity;
import com.mortals.xhx.module.product.model.ProductVersionQuery;
import com.mortals.xhx.module.product.service.ProductService;
import com.mortals.xhx.module.product.service.ProductVersionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.Map;
import java.util.stream.Collectors;
/**
*
* 产品客户端版本
*
* @author zxfei
* @date 2022-08-22
*/
@RestController
@RequestMapping("product/version")
public class ProductVersionController extends BaseCRUDJsonBodyMappingController<ProductVersionService,ProductVersionEntity,Long> {

    @Autowired
    private ProductService productService;

    public ProductVersionController(){
        super.setModuleDesc( "产品客户端版本");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {

        this.addDict(model, "productId", productService.find(new ProductEntity()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getProductName())));

        super.init(model, context);

    }

    @Override
    protected void saveBefore(ProductVersionEntity entity, Map<String, Object> model, Context context) throws AppException {
        ProductVersionEntity productVersionEntity = this.service.selectOne(new ProductVersionQuery().productId(entity.getProductId()).version(entity.getVersion()));
        if(!ObjectUtils.isEmpty(productVersionEntity)){
            entity.setId(productVersionEntity.getId());
            entity.setUpdateUserId(super.getContextUserId(context));
            entity.setUpdateTime(new Date());
        }
        super.saveBefore(entity, model, context);
    }


    /**
     * 发布
     */
    @PostMapping(value = "publish")
    public Rest<Void> producePublish(@RequestBody ProductVersionEntity productVersionEntity) {
        String busiDesc = this.getModuleDesc() + "应用发布";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            ProductVersionEntity entity = this.service.get(productVersionEntity.getId());
            if(ObjectUtils.isEmpty(entity)){
                throw new AppException("应用不存在!");
            }
            this.service.appPublish(productVersionEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("应用发布", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }
}